
%--------------------Begin-Lines to be typed in the command window -------------------------------

%>>[t,Y] = ode45('Xdot',[0 60],[0 0 0 0 0 0])

%>>for figure 5.1
%>>plot(Y(:,1),Y(:,2))
%>>quiver(Y1,X1,V,U)

%>>for figure 5.2
%>> plot(t,Y(:,4),'-',t,Y(:,5),'-.',t,Y(:,6),'.')

%---------------------End-Lines to be typed in the command window -------------------------------

function Xdot=asv(t,y)              %Program for CTC-like algorithm

Xdot    =zeros(6,1);
n       =[y(1) y(2) y(3)]';         %Configuration vector
V       =[y(4) y(5) y(6)]';         %Velocity vector
X       =[n V]';                    %State vector

dhpy    =-0.4;
dhsy    =0.4;                       %distance from the CG to the Sboard & Port side hulls
Lh      =2.54;                      %length of the hull
Bh      =0.127;                     %beam
Th      =0.1524;                    %draft
Ah      =0.6452;                    %hull area

p   =1025;                          %density of sea water
v   =1.4*10^(-6);                   %kinematic viscosity
g   =9.81;

mp=(5*2)*2.2;                       %mass of both pontoons [kg]
Ipz=(1/12)*mp*(Th^2+Bh^2);          %mass moment of inertia about mass-center z-axis [kg*m^2]

%misc. near pontoons
m1  =11*2.2;
m2  =(13*2)*2.2;
ms  =3.6*2.2;                       %mass of strut [kg]
mg  =33*2.2;

%misc inside Gertler body
m3  =4*2.2;
m4  =(11+87)*2.2;
m5  =25*2.2;
m6  =19*2.2;

%total
m   =mp+ms+mg+m1+m2+m3+m4+m5+m6;            %vehicular mass 
Iz  =(3.94e5)*2.2*(0.0254^2);               %(approximate value from pro-E)
Mr  =[m,0,0; 0,m,0; 0,0,Iz];                %mass matrix for the rigid body

% Mah=[-0.0007 0 0;0 7.5673 0.5060;-0.0003 -2.7924 2.6779;];    %hull mass added matrix
% Mas=[0 0 0;0 2768 106.4;0 1835.1 631;];                       %strut mass added matrix
% Mag=[13.8159 0 0;0 149.53 7.8324;0 45.2148 18.7314;];         %Gertler body mass added matrix
% Ma=Mah+Mas+Mag;                                               %Total mass added matrix
% M=Ma+Mr;                                                      %Total mass mtrix

Cr  =[0 -m*y(6) 0;m*y(6) 0 0;0 0 0;];                           %Coriolis & centripetal terms for rigid body
                                  
% Cag=[0 -149.53*y(6) -7.8324*y(6);
%       13.81598*y(6) 0 0;
%       (-13.8159*y(5)+3.454*y(6)) 149.538*y(4) 7.8324*y(4);];  %Coriolis & centripetal terms for added masses of gertler body
%       C=Cr+Cag;% Total c/terms for added masses

J   =[cos(y(3)) -sin(y(3)) 0;sin(y(3)) cos(y(3)) 0;0 0 1;];  %transformation matrix

ndot    =J*V;

Re=(y(4)-dhpy*y(6))*Lh/v;                                   %Renolds Number

if Re>0 & Re<100                                            %at lamina to turbulant transistion point
    Cfhp=0.075/((log(Re)-2)^2);                             %coefficient of friction using ITTC formular
        elseif Re>100.1
            Cfhp=0.075/((log(Re)-2)^2);
                else
                    Cfhp=0;                                 %coefficient of friction using ITTC formular
end
    
Fshpxur=Cfhp*Ah*p*(y(4)-dhpy*y(6))*abs(y(4)-dhpy*y(6));     %skin friction force for port side hull

Fshsxur=Cfhp*Ah*p*(y(4)-dhsy*y(6))*abs(y(4)-dhsy*y(6));     %skin friction force for starboard side hull

Fnpi    = [.25 .3 .35 .4 .45 .5 .55 .6 .65 .7];                     %to be used to find the wave drag (through interpolation)
Cwf     = [1.2 1.75 2.25 2.58 3.75 4.25 4.15 3.85 3.58 3.4]*1e-3;   %The numbers were taken from fig 6 of journal of ship researech june 2001 pg 94
Fnp     = (y(4)-dhpy*y(6))/(g*Lh)^0.5;

for i=1:1:9
    Fnp=(y(4)-dhpy*y(6))/(g*Lh)^0.5;
    if (Fnp>Fnpi(i) & Fnp<Fnpi(i+1))
        Cw=(Cwf(i)+Cwf(i+1))/2;
    else
    Cw=Cwf(i);
    end
end

Frhpxur = Cw*Ah*p*(y(4)-dhpy*y(6))*abs(y(4)-dhpy*y(6));     %wavw drag in Port hull

for i=1:9
    Fnp = (y(4)-dhsy*y(6))/(g*Lh)^0.5;
    if (Fnp>Fnpi(i) & Fnp<Fnpi(i+1))
        Cw = (Cwf(i)+Cwf(i+1))/2;
    else
    Cw = Cwf(i);
    end
end

Frhsxur=Cw*Ah*p*(y(4)-dhpy*y(6))*abs(y(4)-dhsy*y(6));       %wavw drag in Sboard hull

XB  =Lh/2;XS=Lh/2;
CDh =2;                                                     %drag coefficient
dx  =-0.4;                                                  %distance from the CG to the hull centre in Xb direction
VB  =y(5)+(Lh/2+dx)*y(6);                                   %velocity @ bow end
VS  =y(5)-(Lh/2-dx)*y(6);                                   %velocity @ stern end

A1  =1/3*abs(VB-VS)*(VB-VS)/Lh^2*(XB^2*abs(XB)-XS^2*abs(XS));
A2  =0.5*abs(VB-VS)/Lh*y(5)*(XB*abs(XB)-XS*abs(XS));
A3  =0.5*abs(VB-VS)/Lh*abs(y(5))*(XB^2-XS^2);
A4  =y(5)*abs(y(5))*(XB-XS);

Fhpyvr  =0.5*CDh*Th*p*(A1+A2+A3+A4);                        %damping force
Fhsyvr  =0.5*CDh*Th*p*(A1+A2+A3+A4);                        %damping force

mhxur   =(Fshpxur+Frhpxur)*dhpy+(Fshsxur+Frhsxur)*dhsy;     %damping moment

B1  =   1/4*abs(VB-VS)*(VB-VS)/Lh^2*(XB^3*abs(XB)-XS^3*abs(XS));
B2  =   1/3*abs(VB-VS)/Lh*((VB+VS)/2)*(XB^2*abs(XB)-XS^2*abs(XS));
B3  =   1/3*(VB-VS)/Lh*abs((VB+VS)/2)*(XB^3-XS^3);
B4  =   1/2*(VB+VS)/2*abs((VB+VS)/2)*(XB^2-XS^2);
mhpyvr  =   0.5*CDh*Th*p*(B1+B2+B3+B4);
mhsyvr  =   0.5*CDh*Th*p*(B1+B2+B3+B4);
mhuvr   =   mhxur+mhpyvr+mhsyvr;                        %total moment

% Xc=0; Yc=100; Nc=0;
% Tc=[Xc Yc Nc]';                                       %Input/Thrust matrix
%alpa=atan(Yc/Xc);                                      %angle of attack
alpa    =   pi/18;
% alpa=0;;

if alpa>=-180 & alpa<-165
    CL=4.7746*alpa+15;
elseif alpa>=-165 & alpa<-160
    CL=-6.3025*alpa-16.9;
    elseif alpa>=-160 & alpa<-25
        CL=-0.6366*alpa-1.0778;
        elseif alpa>=-25 & alpa<-20
            CL=-9.1673*alpa-4.8;
            elseif alpa>=-20&alpa<20
                CL=4.5837*alpa;
                elseif alpa>=20 & alpa<25
                    CL=-9.1673*alpa+4.8;
                    elseif alpa>=25 & alpa<160
                        CL=-0.6366*alpa+1.0778;
                        elseif alpa>=160 & alpa<165
                            CL=-6.3025*alpa+16.9;
                            else alpa>=165 & alpa<=180
                                CL=4.7746*alpa-15;
                            end
                      
                       
 if alpa>=-180 & alpa<-165
    CD=0;
    elseif alpa>=-165 & alpa<-90
        CD=1.52795*alpa+4.4;  
        elseif alpa>=-90 & alpa<-20
            CD=-1.52795*alpa-0.4;  
            elseif alpa>=-20 & alpa<20
                CD=0;  
                elseif alpa>=20 & alpa<90
                    CD=1.52795*alpa-0.4;
                    elseif alpa>=90 & alpa<165
                        CD=-1.52795*alpa+4.4;  
                        else alpa>=165 & alpa<=180
                            CD=0; 
                        end
                    

Sy  =0.0948;                                          %projected area
Ls  =-1/2*p*abs(y(4)+y(5))*(y(4)+y(5))*Sy*CL;         %lift force
Ds  =1/2*p*abs(y(4)+y(5))*(y(4)+y(5))*Sy*CD;          %Drag force

Fffsxur =   Ls*sin(alpa)+Ds*cos(alpa);
bs  =0.025;                                           %beam
cs  =0.075;
Sx  =0.0395;                                          %projected area
Vsx =y(4)-bs/2*y(6);
Fnsi= [0 0.1 .2  .3  .4  .5   .6   .7   0.8 1.1 2 3]';
Crsi=[0 0   .02 .15 .47 1.12 1.02 .73  0.55  0.23 0.21 0.2]';  
for i=1:11
    Fns=(y(4)-dhsy*y(6))/(g*cs)^0.5;
    if (Fns>Fnsi(i) & Fns<Fnsi(i+1))
        Crs=(Crsi(i)+Crsi(i+1))/2;
    else
    Crs=Crsi(i);
    end
end

frsxur  =1/2*p*Vsx*abs(Vsx)*Sx*Crs;

Fffsyur =Ls*cos(alpa)-Ds*sin(alpa);
Vsy     =y(5)+cs/2*y(6);
Crsy    =2;
frsyur  =1/2*p*Vsy*abs(Vsy)*Sy*Crsy; 
dsy=0;  dsx=0.6;

msuvr = (Fffsxur+frsxur)*dsy+(Fffsyur+frsyur)*dsx;

Sg=2.1565;                                                  % sectional area of the gertler body
Lg=1.875;
Regi= [2 4 6 8 10 12 14 16 18 20 22 24 26 28 30]'*1e6;      % Reynolds number to match w/ Cfg vector (for interpolation)
% Cfg= [3.9 3.4 3.2 3.05 2.825 2.85 2.8 2.725 2.675 2.625 2.56 2.55 2.525 2.5 2.45]'*1e-3;  %Taken from Gertler model 4155 pg 75
Cgi= [3.9 3.4 3.2 3.05 2.9 2.85 2.75 2.7 2.65 2.6 2.55 2.52 2.51 2.49 2.46]'*1e-3;  % Taken from Gertler model 4154 pg 74
for i=1:14
    Reg=y(4)*Lg/v;
    if (Reg>Regi(i) & Reg<Regi(i+1))
        Cdg=(Cgi(i)+Cgi(i+1))/2;
    else
    Cdg=Cgi(i);
    end
end

fgxu=1/2*p*y(4)*abs(y(4))*Sg*Cdg;

for i=[1:1:10]
    dgx     =[0.2006 0.2103 0.2184 0.2260 0.2332 0.2401 0.2467 0.2531 0.2592 0.2651];
    Vgy(i)  =y(5)+dgx(i)*y(6);
    Cgy     =[3.9 3.4 3.2 3.05 2.9 2.85 2.75 2.7 2.65 2.6 2.55 2.52 2.51 2.49 2.46]'*1e-3;
    Cg3d    =[105  59   20.5 9.9 4.4 2.8 1.9 1.4 1.3 1.0 .95 1.0 1.1 1.0 0.2 .24 .54];
    Apg     =[0.0373 0.0391 0.0406 0.0420 0.0434 0.0447 0.0459 0.0471 0.0482 0.0493];
    Fgyvrr(i)   =0.5*p*Apg(i)*abs(Vgy(i))*Vgy(i)*Cgy(i)*Cg3d(i);
    mguvrr(i)   =Fgyvrr(i)*dgx(i);

end  
    Fgyvrr  =   [Fgyvrr(1) Fgyvrr(2) Fgyvrr(3) Fgyvrr(4) Fgyvrr(5) Fgyvrr(6) Fgyvrr(7) Fgyvrr(8) Fgyvrr(9) Fgyvrr(10)];
    mguvrr  =   [mguvrr(1) mguvrr(2) mguvrr(3) mguvrr(4) mguvrr(5) mguvrr(6) mguvrr(7) mguvrr(8) mguvrr(9) mguvrr(10)];

    Fgyvr   =sum(Fgyvrr);
    mguvr1  =sum(mguvrr);
    dgy     =0;
    mguvr   =fgxu*dgy+mguvr1;

DV=[(Fshpxur+Fshsxur+Frhpxur+Frhsxur+Fffsxur+Fffsxur+frsxur+fgxu);
    (Fhpyvr+Fhsyvr+Fffsyur+frsyur);
    (mhuvr+msuvr+mguvr);];
%damping matrix
CC=(Cr*V+DV);

%controller-CTC_Parameters
Kd  =   [10 0 0;0 10 0;0 0 10];
Kp  =   [10 0 0;0 10 0;0 0 10];

A   =10;                                                %desired radius
w   =pi/30;                                             %desred angular speed
nd  =[A*sin(w*t+pi/4); A*cos(w*t+pi/4); -(w*t+pi/4);];  %vector of desired trajectory 

%Velocity
nddot   =A*w*[cos(w*t+pi/4);-sin(w*t+pi/4);-1/A;];

%Accn
ndddot  =A*w*w*[-sin(w*t+pi/4);-cos(w*t+pi/4);0;];

%Error
e   =   nd-n;

%Error rate
edot=nddot-ndot;

%Rotation matrix defferentiation
Jdot=[-sin(y(3))*y(6) -cos(y(3))*y(6) 0;cos(y(3))*y(6) -sin(y(3))*y(6) 0;0 0 0 ;];

%Controller
Vdot=inv(J)*(ndddot+Kd*edot+Kp*e-Jdot*V);
Tc  =Mr*Vdot+CC;

%SS representation
fx  =[ndot;  (-inv(Mr)*CC);];
B   =[zeros(3); inv(Mr);];
Xdot=fx+B*Tc;


