
%--------------------Begin-Lines to be typed in the command window -------------------------------

% >> options = odeset('RelTol',1e-1,'AbsTol',[1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1 1e-1]);
% >> [t,Y] = ode45('control_NN1',[0:0.1:40],[0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0],options);

%---------------------End-Lines to be typed in the command window -------------------------------

function Xdot=control_nn(t,y)	%program for path tracking: CTC+NN control

n   =[y(1) y(2) y(3)]';                                         %Configuration vector=[x y ep]
V   =[y(4) y(5) y(6)]';                                         %Velocity vector=[u v r]
J   =[cos(y(3)) -sin(y(3)) 0;sin(y(3)) cos(y(3)) 0;0 0 1;];	    %rotation matrix
ndot=J*V;	                                                    %velocities w.r.t. earth fixed frame

dhpy    =-0.4;
dhsy    =0.4;	                                %distance from the CG to the Sboard & Port side hulls
Lh      =2.54;	                                %length of the hull
Bh      =0.127;	                                %beam
Th      =0.1524;	                            %draft
Ah      =0.6452;	                            %hull area

p   =1025;                                  %density of sea water
vis =1.4*10^(-6);		                    %kinematic viscosity
g   =9.81;

mp  =(5*2)*2.2;                             %mass of both pontoons [kg]
Ipz =(1/12)*mp*(Th^2+Bh^2);                 %mass moment of inertia about mass-center z-axis [kg*m^2]

%misc. near pontoons
m1  =11*2.2;
m2  =(13*2)*2.2;
ms  =3.6*2.2;                               %mass of strut [kg]
mg  =33*2.2;	                            %misc inside Gertler body
m3  =4*2.2;
m4  =(11+87)*2.2;
m5  =25*2.2;
m6  =19*2.2;

m   =mp+ms+mg+m1+m2+m3+m4+m5+m6;            %total vehicular mass 
Iz  =(3.94e5)*2.2*(0.0254^2);          	    %(approximate value from pro-E)
M   =[m,0,0; 0,m,0; 0,0,Iz];    	        %mass matrix for the rigid body

C   =[0 -m*y(6) 0;m*y(6) 0 0;0 0 0;];		%Coriolis & centripetal terms for rigid body
Cr  =C*V;				                    %Coriolis terms with velocity

Re  =(y(4)-dhpy*y(6))*Lh/vis; 		        %Renolds Number

if Re>0 & Re<100    	                    %at lamina to turbulant transistion point
    Cfhp    =0.075/((log(Re)-2)^2); 	    %coefficient of friction using ITTC formular
elseif Re>100
    Cfhp    =0.075/((log(Re)-2)^2);
else
    Cfhp    =0; 	                        %coefficient of friction using ITTC formular
end
    
Fshpxur =   Cfhp*Ah*p*(y(4)-dhpy*y(6))*abs(y(4)-dhpy*y(6));
%skin friction force for port side hull
Fshsxur =   Cfhp*Ah*p*(y(4)-dhsy*y(6))*abs(y(4)-dhsy*y(6));
%skin friction force for starboard side hull

Fnpi    = [.25 .3 .35 .4 .45 .5 .55 .6 .65 .7];                     %to be used to find the wave drag (through interpolation)
Cwf     = [1.2 1.75 2.25 2.58 3.75 4.25 4.15 3.85 3.58 3.4]*1e-3;   %The numbers were taken from fig 6 of journal of ship researech june 2001 pg 94
Fnp     = (y(4)-dhpy*y(6))/(g*Lh)^0.5;

for i=1:9
    Fnp =   (y(4)-dhpy*y(6))/(g*Lh)^0.5;
    if (Fnp>Fnpi(i) & Fnp<Fnpi(i+1))
        Cw  =   (Cwf(i)+Cwf(i+1))/2;
    else
    Cw  =   Cwf(i);
    end
end
Frhpxur =   Cw*Ah*p*(y(4)-dhpy*y(6))*abs(y(4)-dhpy*y(6));           %wavw drag in Port hull

for i=1:9
    Fnp =(y(4)-dhsy*y(6))/(g*Lh)^0.5;
    if (Fnp>Fnpi(i) & Fnp<Fnpi(i+1))
        Cw  =(Cwf(i)+Cwf(i+1))/2;
    else
        Cw  =Cwf(i);
    end
end
Frhsxur =Cw*Ah*p*(y(4)-dhpy*y(6))*abs(y(4)-dhsy*y(6));              %wavw drag in Sboard hull

XB  =Lh/2;XS=Lh/2;
CDh =2;                                                             %drag coefficient
dx  =-0.4;                                                          %distance from the CG to the hull centre in Xb direction
VB  =y(5)+(Lh/2+dx)*y(6);                                           %velocity @ bow end
VS  =y(5)-(Lh/2-dx)*y(6);                                           %velocity @ stern end

A1  =1/3*abs(VB-VS)*(VB-VS)*(XB^2*abs(XB)-XS^2*abs(XS))/Lh^2;
A2  =0.5*abs(VB-VS)*y(5)*(XB*abs(XB)-XS*abs(XS))/Lh;
A3  =0.5*abs(VB-VS)*abs(y(5))*(XB^2-XS^2)/Lh;
A4  =y(5)*abs(y(5))*(XB-XS);
Fhpyvr  =0.5*CDh*Th*p*(A1+A2+A3+A4);                                %damping force
Fhsyvr  =0.5*CDh*Th*p*(A1+A2+A3+A4);                                %damping force

mhxur   =(Fshpxur+Frhpxur)*dhpy+(Fshsxur+Frhsxur)*dhsy;             %damping moment

B1  =1/4*abs(VB-VS)*(VB-VS)/Lh^2*(XB^3*abs(XB)-XS^3*abs(XS));
B2  =1/3*abs(VB-VS)/Lh*((VB+VS)/2)*(XB^2*abs(XB)-XS^2*abs(XS));
B3  =1/3*(VB-VS)/Lh*abs((VB+VS)/2)*(XB^3-XS^3);
B4  =1/2*(VB+VS)/2*abs((VB+VS)/2)*(XB^2-XS^2);
mhpyvr  =0.5*CDh*Th*p*(B1+B2+B3+B4);
mhsyvr  =0.5*CDh*Th*p*(B1+B2+B3+B4);
mhuvr   =mhxur+mhpyvr+mhsyvr;                                       %total moment

alpa    =pi/18; CL  =4.5837*alpa; CD    =0.1;  
                                   
Sy  =0.0948;                                                        %projected area
Ls  =-1/2*p*abs(y(4)+y(5))*(y(4)+y(5))*Sy*CL;                       %lift force
Ds  =1/2*p*abs(y(4)+y(5))*(y(4)+y(5))*Sy*CD;                        %Drag force

Fffsxur =Ls*sin(alpa)+Ds*cos(alpa);
bs      =0.025;                                                     %beam
cs      =0.075;
Sx      =0.0395;                                                    %projected area
Vsx     =y(4)-bs/2*y(6);
Fnsi    = [0 0.1 .2  .3  .4  .5   .6   .7   0.8 1.1 2 3]';
Crsi    =[0 0   .02 .15 .47 1.12 1.02 .73  0.55  0.23 0.21 0.2]';  

for i=1:11
        Fns =(y(4)-dhsy*y(6))/(g*cs)^0.5;
    if (Fns>Fnsi(i) & Fns<Fnsi(i+1))
        Crs =(Crsi(i)+Crsi(i+1))/2;
        else
            Crs =Crsi(i);
        end
end
frsxur  =1/2*p*Vsx*abs(Vsx)*Sx*Crs;

Fffsyur =Ls*cos(alpa)-Ds*sin(alpa);
Vsy     =y(5)+cs/2*y(6);
Crsy    =2;
frsyur  =1/2*p*Vsy*abs(Vsy)*Sy*Crsy; 
dsy=0;  dsx=0.6;

msuvr   =(Fffsxur+frsxur)*dsy+(Fffsyur+frsyur)*dsx;

Sg  =2.1565;                                                                             %sectional area of the Gertler body
Lg  =1.875;
Regi= [2 4 6 8 10 12 14 16 18 20 22 24 26 28 30]'*1e6;                                   %Reynolds number to match w/ Cfg vector (for interpolation)
% Cfg= [3.9 3.4 3.2 3.05 2.825 2.85 2.8 2.725 2.675 2.625 2.56 2.55 2.525 2.5 2.45]'*1e-3;  %Taken from Gertler model 4155 pg 75
Cgi = [3.9 3.4 3.2 3.05 2.9 2.85 2.75 2.7 2.65 2.6 2.55 2.52 2.51 2.49 2.46]'*1e-3;      %Taken from Gertler model 4154 pg 74

for i=1:14
    Reg =y(4)*Lg/vis;
    if (Reg>Regi(i) & Reg<Regi(i+1))
        Cdg =(Cgi(i)+Cgi(i+1))/2;
    else
    Cdg =Cgi(i);
    end
end

fgxu    =1/2*p*y(4)*abs(y(4))*Sg*Cdg;

for i=1:10
    dgx         =[0.2006 0.2103 0.2184 0.2260 0.2332 0.2401 0.2467 0.2531 0.2592 0.2651];
    Vgy(i)      =y(5)+dgx(i)*y(6);
    Cgy         =[3.9 3.4 3.2 3.05 2.9 2.85 2.75 2.7 2.65 2.6 2.55 2.52 2.51 2.49 2.46]'*1e-3;
    Cg3d        =[105  59   20.5 9.9 4.4 2.8 1.9 1.4 1.3 1.0 .95 1.0 1.1 1.0 0.2 .24 .54];
    Apg         =[0.0373 0.0391 0.0406 0.0420 0.0434 0.0447 0.0459 0.0471 0.0482 0.0493];
    Fgyvrr(i)   =0.5*p*Apg(i)*abs(Vgy(i))*Vgy(i)*Cgy(i)*Cg3d(i);
    mguvrr(i)   =Fgyvrr(i)*dgx(i);

end  
    Fgyvrr  =[Fgyvrr(1) Fgyvrr(2) Fgyvrr(3) Fgyvrr(4) Fgyvrr(5) Fgyvrr(6) Fgyvrr(7) Fgyvrr(8) Fgyvrr(9) Fgyvrr(10)];
    mguvrr  =[mguvrr(1) mguvrr(2) mguvrr(3) mguvrr(4) mguvrr(5) mguvrr(6) mguvrr(7) mguvrr(8) mguvrr(9) mguvrr(10)];
    Fgyvr   =sum(Fgyvrr);
    mguvr1  =sum(mguvrr);
    dgy     =0;
    mguvr   =fgxu*dgy+mguvr1;

DV  =[(Fshpxur+Fshsxur+Frhpxur+Frhsxur+Fffsxur+Fffsxur+frsxur+fgxu);
    (Fhpyvr+Fhsyvr+Fffsyur+frsyur);
    (mhuvr+msuvr+mguvr);];

dpx =0.85;
hV1 =[DV(1) DV(2) DV(3)]'+Cr;
hV  =-[hV1(1) hV1(2) hV1(3)/dpx]';

%distance from CG to propellers in X-direction
M_new   =[m 0 0;0 m 0;0 0 Iz/dpx;];                            %Mass matrix required


% Desired Profile
A       =10;
w       =pi/10;
x_d     =2*A*sin(w*t/2);
xdot_d  =A*w*cos(w*t/2);
xddot_d =-0.5*A*w*w*sin(w*t/2);

y_d     =A*sin(w*t);
ydot_d  =A*w*cos(w*t);
yddot_d =-A*w*w*sin(w*t);
 
%error
ex = x_d-y(1);
ey = y_d-y(2);

%error rate
exdot = xdot_d-ndot(1);
eydot = ydot_d-ndot(2);

%waights update law parameters
r_x =ex+exdot;
Kx  =0.0055;
E_x =sqrt(ex^2+exdot^2);
mu_x=120;%5*exp(-ex^2+exdot^2);

r_y =ey+eydot;
Ky  =0.0055;
E_y =sqrt(ey^2+eydot^2);
mu_y=120;

if (t>0)
    xddot = gradient(ndot(1));
else
    xddot = 0;
end

wx  = [y(7) y(8) y(9) y(10) y(11) y(12) y(13)]';                %weights; x-net
wy  = [y(14) y(15) y(16) y(17) y(18) y(19) y(20)]';             %weights; y-net

zx(1)=ndot(1);     zx(2)=ndot(2);     zx(3)=ndot(3);     zx(4)=xddot;
%NN output in longitudianl direction
r1=7;                                                           %no of neurones

for i=1:7
    for j=1:4
           
        if j==1
            cx(i,j)     = rand(1)+5;
            sigmax(i,j) =25;
        else
            cx(i,j)     = -5+2*rand(1)/2147483647;
            sigmax(i,j) = 2;
        end
              
        xsssi(j)=((zx(j)-cx(i,j))^2)/(2*sigmax(i,j)^2);
    end
    xssi    =[xsssi(1) xsssi(2) xsssi(3) xsssi(4)];
    xsi(i)  =exp(-sum(xssi));
end

six     =[xsi(1) xsi(2) xsi(3) xsi(4) xsi(5) xsi(6) xsi(7)]';
vx      =wx'*six;
wxdot   =mu_x*r_x*six-Kx*E_x*wx;

%NN output in lateral direction
zy(1)=ndot(1); zy(2)=ndot(3); zy(3)=ndot(3); zy(4)=0;
r2=7;                                                       %no of neurones

for i=1:7
    for j=1:4
               
        if j==2
            cy(i,j)=rand(1)+5;
            sigmay(i,j)=8;
        else
            cy(i,j)     =-5+2*rand(1)/2147483647;
            sigmay(i,j) =2;
        end
               
        ysssi(j)=((zy(j)-cy(i,j))^2)/(2*sigmay(i,j)^2);
    end
    yssi=[ysssi(1) ysssi(2) ysssi(3) ysssi(4)];
    ysi(i)=exp(-sum(yssi));
end

siy     =[ysi(1) ysi(2) ysi(3) ysi(4) ysi(5) ysi(6) ysi(7)]';
vy      =wy'*siy;
wydot   =mu_y*r_y*siy-Ky*E_y*wy;
wdot    =[wxdot;wydot;];                                        %weight matrix

%Controller parameters
Kpx=1; Kvx=7; Kpy=1; Kvy=7; kw_x=0.25; kw_y=0.25; 
M_new_xy=[m 0;0 m];

acc_x   =xddot_d+Kvx*exdot+Kpx*ex+kw_x*vx;
acc_y   =yddot_d+Kvy*eydot+Kpy*ey+kw_y*vy;

nddot_xy=[acc_x acc_y]';

J_xy    =[cos(y(3)) -sin(y(3)); sin(y(3)) cos(y(3));];
Jdot_xy =[-sin(y(3)) -cos(y(3));cos(y(3)) -sin(y(3));];    
V_xy    =[y(4) y(5)]'; M_xy =[m 0;0 m;]; h_xy =[hV(1) hV(2)]';

Vdot_xy =inv(J_xy)*(nddot_xy-Jdot_xy*V_xy);
g_xy    =M_xy*Vdot_xy-h_xy;
gx      =g_xy(1); gye=g_xy(2);
gU      =[gx gye gye]';
Vdot    =inv(M_new)*(hV+gU);
Thrust  =sqrt(gx^2+gye^2);                                      %thrust
delta   =atan(gye/gx);                                          %steering angle

Xdot=[ndot;Vdot;wdot;];

