#include<stdio.h>
#include<math.h>
#include<stdlib.h>


#define pi 3.141592653589793238
#define g 9.80665


void control( float endTime, float t, float vel_x_d, float acc_x_d, float vel_x, float vel_y_s, float vel_yaw_r,
              float acc_x[], float acc_y_s[], float acc_yaw_r[], float T[], float steer[] ){

/* Computed torque controller */

        float h_x, h_y_s;
        float r1, r2, r3, r4;
        float xi1, xi2;
 		float g_x, g_y_yaw;

/* ** Model Parameters (From SAVRIN Data-sheet and k1&k2 guessed according to the book: Gillespie, "THEORY OF Ground Vihicles")** */

	float m = 1760.0;			// Curb weight (1640kg) + 2 X 60kg pasenger
	float I_z = 3332.0;			// 3105 * 1760/1640
	float Cf = 131391.0;
    float Cr = 115669.0;
	float a = 1.193;
	float b = 1.587;
        float k2 = 0.49;		// 0.29 Toyota Prius
        float k1 = 0.008;		// 0.0035
        float f = 0.02;			// P175/65R14 Low rolling resistance tires
        float d_s = 1.193 + 0.5 * 1.82;
        float rho;

        float vel_theta;
        float acc_theta;

        // ROAD PROFILE

        if ( t <= 4.0){
                rho = 0.0;                                // Straight road
        }
                else if ( t > 4.0 && t <= endTime){
                        rho = 1.0/600.0;                 // Road -- Right curvature of radius 6000 m
                }


        r1 = m * a/(I_z + m * d_s * a);
        r2 = (Cr/I_z) * (a * (I_z - m*d_s*b)/(I_z + m*d_s*a) + b) * (1 - a*m*(b+d_s)/(I_z+m*d_s*a));
        r3 = (Cr * (b+d_s)/I_z) * (a * (I_z -m*d_s*b)/(I_z+m*d_s*a) + b);
        r4 = -rho * b * Cr * (b + a * (I_z -m*d_s*b)/(I_z+m*d_s*a))/I_z;

        xi1= (r1 * pow(vel_x,2) - r3)/vel_x;
        xi2= rho * r1 * pow(vel_x,2) + r4;

        h_x  =  m * (vel_y_s - rho * d_s * vel_x) * vel_yaw_r + (f * k2 * a/(a+b) - k2) * pow(vel_x,2)
                + m * rho * vel_x * vel_y_s - d_s * pow(vel_yaw_r,2);
        h_y_s= -m * vel_x * vel_yaw_r - Cr*(1 - m*d_s*b/I_z) * (vel_y_s + (b+d_s)*vel_yaw_r)/vel_x
                - m * rho * pow(vel_x,2) + rho * b * Cr*(1 - m*d_s*b/I_z);

        vel_theta = vel_yaw_r - r1 * vel_y_s;

        
		// COMPUTE CONTROL I/Ps

        float k_x        = 20.0;
        float k_theta    = 21.0;
        float k1_theta;
        float k_theta1   = 10.0 * k_theta;
        float k_y_s      = 25.0;
        float kappa      = 0.04;
        float varepsilon = 1.2;         
        float sign_y_s;
        float T_capa;

        if ( vel_y_s >= 0) {
                sign_y_s = 1.0;
        }
                else {
                        sign_y_s = -1.0;
                }

        // K'_theta

        if ( vel_y_s > kappa) {
                k1_theta = k_theta;
        }
                else {
                        k_theta  = k_theta1;
                        k1_theta = k_theta1 * abs(vel_y_s)/(varepsilon * kappa);
                }

        // K_y_s

        if ( vel_y_s > kappa) {
                k_y_s = k_y_s;
        }
                else {
                        k_y_s = 2.0 + (k_y_s - 2.0) * exp(-10.0 * abs(vel_theta));
                }


        g_x     = m * (acc_x_d + k_x * (vel_x_d - vel_x)) - h_x;
        g_y_yaw = -(I_z * r1/a) * (h_y_s/m + k1_theta * pow(r1,2) * vel_x * vel_theta
                  + k1_theta * r2 * vel_theta /vel_x + k_y_s * vel_y_s / (1+pow(vel_y_s,2))
                  + sign_y_s * k_theta * abs(xi1) * pow(vel_theta,2)/kappa
                  + sign_y_s * k_theta * abs(xi2) * abs(vel_theta)/kappa);

        steer[0] = (g_y_yaw * Cf*(vel_y_s - d_s*vel_yaw_r + a*(vel_yaw_r + rho*vel_x))/vel_x)/(Cf + g_x);
        T[0]     = g_x*cos(steer[0]) + g_y_yaw*sin(steer[0]) + f*b*(m*g - k1*pow(vel_x,2))/(a+b);
        //T[0]     = g_x + g_y_yaw*steer[0] + f*b*(m*g - k1*pow(vel_x,2))/(a+b);

         
		// I choose 12.0 m/s as the bound for validity of the equation according to the diagram in Gillespie book

        if ( vel_x > 12.0 ){
                T_capa = 110250.0/vel_x;  // From F=P/vel = 110.25*1000.0/vel ; Traction-force capability
        }                                 // Engine power is 150 PS = 150 * 0.735 kW = 110.25 kW
                else {
                        T_capa = 3.0 * 1760.0;
                }

        if ( T[0] > T_capa ){
                T[0]  = T_capa;
        }
                else {
                        T[0] = T[0];
                }

        if ( steer[0] > 40.0*pi/180.0 ){
             steer[0] = 40.0*pi/180.0;
        }
                else if ( steer[0] < -40.0 * pi/180.0 ){
                          steer[0] = - 40.0*pi/180.0;
                }
                        else {
                        steer[0] = steer[0];
                        }
                        
        if ( t == 0.0){
                T[0]     = 375.0;
                steer[0] = 0.0;
        }
                else {
                        T[0]     = T[0];
                        steer[0] = steer[0];
                }  

        // DYNAMIC SIMULATION

        if (t>1.0){
        g_x     = (T[0] - f*b*(m*g - k1*pow(vel_x,2))/(a+b))*cos(steer[0]) + Cf*((vel_y_s - d_s*vel_yaw_r
                + a*(vel_yaw_r + rho*vel_x))/vel_x - steer[0])*sin(steer[0]);
        g_y_yaw = (T[0] - f*b*(m*g - k1*pow(vel_x,2))/(a+b))*sin(steer[0]) - Cf*((vel_y_s - d_s*vel_yaw_r
                + a*(vel_yaw_r + rho*vel_x))/vel_x - steer[0])*cos(steer[0]);
        }
                else {
                g_x = g_x;
                g_y_yaw = g_y_yaw;
                }

        acc_x[0]   = (g_x + h_x)/m;
        acc_y_s[0] = h_y_s/m + (a * g_y_yaw)/(I_z * r1);
        acc_theta  = xi1 * vel_theta + pow(r1,2) * vel_x * vel_y_s + r2 * vel_y_s / vel_x + xi2;

        acc_yaw_r[0] = acc_theta + r1 * acc_y_s[0];        
}


int main( void )
{
   int i,j,k;
   float dt;                            // sampling interval
   float t;                             // current time
   float endTime;
   float acc_x[1], acc_y_s[1], acc_yaw_r[1];// Second time derivatives of x, y_s, rel-yaw
   int totalSteps;
   float T[1];                         // Traction and/or breaking force
   float x;                            // Longitudinal position of the vehicle
   float vel_x;                        // Longitudinal Speed of the vehicle
   float y_s;                          // Lateral sensor error
   float yaw_r;                        // Relative yaw
   float vel_y_s;                      // Lateral sensor error rate
   float vel_yaw_r;                    // Yaw rate
   float vel_x_d;                      // Desired Velocity
   float acc_x_d;                      // Desired Acceleration
   float vel_x_d0;                     // Initial desired velocity
   float steer[1];                     // Steering angle


       	FILE *fp;

	if ((fp=fopen("data.dat","w"))==NULL){

		printf("Can't open File\n");
		exit(1);
	}

/*        FILE *fp1;

	if ((fp1=fopen("SteerNN.dat","w"))==NULL){

		printf("Can't open File\n");
		exit(1);
	}   */


   dt = 0.001;

   endTime = 20.0;
   totalSteps = (int)(endTime/dt);

 /* ********** initial conditions ********** */

 // Positions
        x     = -0.5;
        y_s   = 0.0;
        yaw_r = 0.0;

 // Velocities
        vel_x     = 20.0;
        vel_y_s   = 0.05;//0.3;
        vel_yaw_r = 0.01;//0.07;
        vel_x_d0  = 20.0;

 // Accelerations
        acc_x[0]     = 0.0;
        acc_y_s[0]   = 0.0;
        acc_yaw_r[0] = 0.0;

    /* ****************************** CONTROL LOOP ****************************** */

   for (j = 0; j <= totalSteps; j++) {

	       t = dt * j;

    /* ******************** Begin: Desired velocity profile ********************** */

                if ( t <= 2.0 ){
                        vel_x_d = 20.0;
                }
                        else if ( t > 2.0 && t <= 7.0 ){
                                vel_x_d = -0.4 * (t-2.0) + 20.0;
                        }
                                else {
                                        vel_x_d = 18.0;
                                }

                acc_x_d = (vel_x_d - vel_x_d0)/dt;
                vel_x_d0= vel_x_d;

	/* ********************** End: Desired velocity profile ************************ */

	control( endTime, t, vel_x_d, acc_x_d, vel_x, vel_y_s, vel_yaw_r, acc_x, acc_y_s, acc_yaw_r, T, steer );

 /*       if ( j % 1000 == 0) {
                fprintf(fp1, "%2f %4f \n", t, steer[0]);
        }       */

         /* Euler method to get positions and velocities */

         if ( t == 0.0){
                x         = x;
                y_s       = y_s;
                yaw_r     = yaw_r;

                vel_x     = vel_x;
                vel_y_s   = vel_y_s;
                vel_yaw_r = vel_yaw_r;
         }

                else {
                        vel_x     = vel_x     + acc_x[0]     * dt;
                        vel_y_s   = vel_y_s   + acc_y_s[0]   * dt;
                        vel_yaw_r = vel_yaw_r + acc_yaw_r[0] * dt;

                        x      = x     + vel_x     * dt     + acc_x[0] * pow(dt,2)/2.0;
                        y_s    = y_s   + vel_y_s   * dt   + acc_y_s[0] * pow(dt,2)/2.0;
                        yaw_r  = yaw_r + vel_yaw_r * dt + acc_yaw_r[0] * pow(dt,2)/2.0;
                }

/*        if ( j % 1000 == 0) {
                fprintf(fp, "%2f %4f %4f %4f %4f %4f %4f %4f %4f %4f %4f %4f\n",t,e[0],e[1],e[2], vel_x, acc_x[0],
                vel_y_s, acc_y_s[0], vel_yaw_r, acc_yaw_r[0], vel_yaw_d, T[0]);
        }  */

           if ( j % 100 == 0) {
                fprintf(fp, "%2f %4f %4f %4f %4f %4f %4f \n",t, vel_x_d, vel_x, vel_y_s, vel_yaw_r, T[0], steer[0]);
           }

}
       fclose(fp);
       return 0;
}

